@@ -34,8 +34,8 @@ BinaryClock::BinaryClock(QObject *parent, const QVariantList &args)
       m_showGrid(true),
       m_time(0, 0)
 {
-    KGlobal::locale()->insertCatalog("libplasmaclock");
-    KGlobal::locale()->insertCatalog("timezones4");
+    KGlobal::locale()->insertCatalog(QLatin1String("libplasmaclock"));
+    KGlobal::locale()->insertCatalog(QLatin1String("timezones4"));
 
     setHasConfigurationInterface(true);
     resize(getWidthFromHeight(128), 128);
@@ -44,16 +44,7 @@ BinaryClock::BinaryClock(QObject *parent, const QVariantList &args)
 void BinaryClock::init()
 {
     ClockApplet::init();
-    KConfigGroup cg = config();
-    m_showSeconds = cg.readEntry("showSeconds", m_showSeconds);
-    m_showGrid = cg.readEntry("showGrid", m_showGrid);
-    m_showOffLeds = cg.readEntry("showOffLeds", m_showOffLeds);
-
-    m_customOnLedsColor = cg.readEntry("customOnLedsColor", false);
-    m_customOffLedsColor = cg.readEntry("customOffLedsColor", false);
-    m_customGridColor = cg.readEntry("customGridColor", false);
-
-    setCurrentTimezone(cg.readEntry("timezone", localTimezone()));
+    clockConfigChanged();
 
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), SLOT(updateColors()));
 
@@ -105,7 +96,7 @@ void BinaryClock::constraintsEvent(Plasma::Constraints constraints)
 
 void BinaryClock::connectToEngine()
 {
-    Plasma::DataEngine* timeEngine = dataEngine("time");
+    Plasma::DataEngine* timeEngine = dataEngine(QLatin1String("time"));
 
     if (m_showSeconds) {
         timeEngine->connectSource(currentTimezone(), this, 500);
@@ -118,7 +109,7 @@ void BinaryClock::dataUpdated(const QString& source, const Plasma::DataEngine::D
 {
     Q_UNUSED(source);
 
-    m_time = data["Time"].toTime();
+    m_time = data[QLatin1String("Time")].toTime();
 
     if (m_time.minute() == m_lastTimeSeen.minute() &&
         m_time.second() == m_lastTimeSeen.second()) {
@@ -140,7 +131,7 @@ void BinaryClock::createClockConfigurationInterface(KConfigDialog *parent)
 {
     QWidget *widget = new QWidget();
     ui.setupUi(widget);
-    parent->addPage(widget, i18n("Appearance"), "view-media-visualization");
+    parent->addPage(widget, i18n("Appearance"), QLatin1String("view-media-visualization"));
 
     ui.showSecondHandCheckBox->setChecked(m_showSeconds);
     ui.showGridCheckBox->setChecked(m_showGrid);
@@ -180,15 +171,15 @@ void BinaryClock::clockConfigAccepted()
     m_customGridColor = ui.gridCustomColorRadioButton->isChecked();
 
     if (m_customOnLedsColor){
-         m_onLedsColor = ui.onLedsCustomColorButton->color();
+        m_onLedsColor = ui.onLedsCustomColorButton->color();
     }
 
     if (m_customOffLedsColor){
-         m_offLedsColor = ui.offLedsCustomColorButton->color();
+        m_offLedsColor = ui.offLedsCustomColorButton->color();
     }
 
     if (m_customGridColor){
-         m_gridColor = ui.gridCustomColorButton->color();
+        m_gridColor = ui.gridCustomColorButton->color();
     }
 
     cg.writeEntry("showSeconds", m_showSeconds);
@@ -203,21 +194,32 @@ void BinaryClock::clockConfigAccepted()
     cg.writeEntry("offLedsColor", ui.offLedsCustomColorButton->color());
     cg.writeEntry("gridColor", ui.gridCustomColorButton->color());
 
-    dataEngine("time")->disconnectSource(currentTimezone(), this);
+    dataEngine(QLatin1String("time"))->disconnectSource(currentTimezone(), this);
     connectToEngine();
 
-    updateColors();
-
-    constraintsEvent(Plasma::AllConstraints);
     update();
     emit configNeedsSaving();
 }
 
+void BinaryClock::clockConfigChanged()
+{
+    KConfigGroup cg = config();
+    m_showSeconds = cg.readEntry("showSeconds", m_showSeconds);
+    m_showGrid = cg.readEntry("showGrid", m_showGrid);
+    m_showOffLeds = cg.readEntry("showOffLeds", m_showOffLeds);
+
+    m_customOnLedsColor = cg.readEntry("customOnLedsColor", false);
+    m_customOffLedsColor = cg.readEntry("customOffLedsColor", false);
+    m_customGridColor = cg.readEntry("customGridColor", false);
+
+    updateColors();
+}
+
 void BinaryClock::changeEngineTimezone(const QString &oldTimezone, const QString &newTimezone)
 {
-    dataEngine("time")->disconnectSource(oldTimezone, this);
+    dataEngine(QLatin1String("time"))->disconnectSource(oldTimezone, this);
 
-    Plasma::DataEngine* timeEngine = dataEngine("time");
+    Plasma::DataEngine* timeEngine = dataEngine(QLatin1String("time"));
     if (m_showSeconds) {
         timeEngine->connectSource(newTimezone, this, 500);
     } else {
