@@ -22,13 +22,14 @@
 
 #include <QString>
 #include <QStringList>
-#include <QImage>
 #include <QObject>
 
 #include <KUrl>
 
 #include <Plasma/DataEngine>
 
+#include "picture.h"
+
 class QTimer;
 
 class SlideShow : public QObject
@@ -42,38 +43,41 @@ public:
     void setDirs(const QStringList &slideShowPaths, bool recursive = false);
     void setImage(const QString &imagePath);
     void setRandom(bool);
-    QImage image();
+    QImage image() const;
     KUrl currentUrl() const;
-
+    QString message() const;
+    void updateImage(QString);
     void setUpdateInterval(int msec);
 
 public Q_SLOTS:
     void dataUpdated(const QString &name, const Plasma::DataEngine::Data &data);
+    void firstPicture();
+    void nextPicture();
+    void previousPicture();
 
 Q_SIGNALS:
     void pictureUpdated();
+    QString emptyDirMessage();
+
+private Q_SLOTS:
+    void clearPicture();
+    void pictureLoaded(QImage image);
 
 private:
     void addImage(const QString &imagePath);
-    void addDir(const QString &path);
-    void addRecursiveDir(const QString &path);
-    KUrl url();
+    void addDir(const QString &path, bool recursive);
+    KUrl url(int offset = 1);
 
-private Q_SLOTS:
-    void updatePicture();
-    void clearPicture();
-
-private:
     QStringList m_picturePaths;
     QStringList m_filters;
     int m_slideNumber;
     bool m_useRandom;
 
     QList<int> m_indexList;
-    int m_randomInt;
     KUrl m_currentUrl;
     QTimer *m_timer;
-    QImage m_picture;
+    QImage m_image;
+    Picture *m_picture;
 };
 
 #endif /*SLIDESHOW_H*/
