@@ -1,5 +1,6 @@
 /*
- *   Copyright (C) 2008 Christian Weilbach <dunsens@web.de>
+ *   Copyright (C) 2008-2010 Christian Weilbach <dunsens@web.de>
+ *   Copyright (C) 2010 Ruslan Nigmatullin <euroelessar@ya.ru>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License version 2 as
@@ -48,13 +49,17 @@
 
 IncomingMsg::IncomingMsg(QObject *parent, const QVariantList &args)
         : Plasma::Applet(parent, args),
-        mEvolutionLabel(0), mEvolutionIconLabel(0),
-        mKMailLabel(0), mKMailIconLabel(0),
-        mXChatLabel(0), mXChatIconLabel(0),
-        mKopeteLabel(0), mKopeteIconLabel(0),
-        mPidginLabel(0), mPidginIconLabel(0),
-        mLayout(0), mEvolutionLayout(0), mKMailLayout(0), mXChatLayout(0), mKopeteLayout(0),
-        mPidginLayout(0)
+          mEvolutionLabel(0), mEvolutionIconLabel(0),
+          mKMailLabel(0), mKMailIconLabel(0),
+          mXChatLabel(0), mXChatIconLabel(0),
+          mKopeteLabel(0), mKopeteIconLabel(0),
+          mPidginLabel(0), mPidginIconLabel(0),
+          mQutIMLabel(0), mQutIMIconLabel(0),
+          mErrorLabel(0), mLayout(0),
+          mEvolutionLayout(0), mKMailLayout(0),
+          mXChatLayout(0), mKopeteLayout(0),
+          mPidginLayout(0), mQutIMLayout(0),
+          mQutIUnreadCount(0)
 {
     // this will get us the standard applet background, for free!
     setBackgroundHints(DefaultBackground);
@@ -78,53 +83,34 @@ IncomingMsg::~IncomingMsg()
     delete mPidginLayout;
     delete mPidginIconLabel;
     delete mPidginLabel;
+    
+    delete mQutIMLayout;
+    delete mQutIMIconLabel;
+    delete mQutIMLabel;
 }
 
 void IncomingMsg::init()
 {
     /* initialize layout */
     setHasConfigurationInterface(true);
+
+    configChanged();
+
+    initLayout();
+}
+
+void IncomingMsg::configChanged()
+{
     KConfigGroup cg = config();
     mShowKMail = cg.readEntry("showKMail", true);
     mShowXChat = cg.readEntry("showXChat", true);
     mShowKopete = cg.readEntry("showKopete", true);
     mShowPidgin = cg.readEntry("showPidgin", true);
-
-    initLayout();
+    mShowQutIM = cg.readEntry("showQutIM", true);
 }
 
-void IncomingMsg::initLayout()
+void IncomingMsg::initEvolutionLayout()
 {
-    delete mKMailLayout;
-    mKMailLayout = 0;
-    delete mKMailIconLabel;
-    mKMailIconLabel = 0;
-    delete mKMailLabel;
-    mKMailLabel = 0;
-
-    delete mXChatLayout;
-    mXChatLayout = 0;
-    delete mXChatIconLabel;
-    mXChatIconLabel = 0;
-    delete mXChatLabel;
-    mXChatLabel = 0;
-
-    delete mKopeteLayout;
-    mKopeteLayout = 0;
-    delete mKopeteIconLabel;
-    mKopeteIconLabel = 0;
-    delete mKopeteLabel;
-    mKopeteLabel = 0;
-
-    delete mPidginLayout;
-    mPidginLayout = 0;
-    delete mPidginIconLabel;
-    mPidginIconLabel = 0;
-    delete mPidginLabel;
-    mPidginLabel = 0;
-
-    mLayout = new QGraphicsLinearLayout(Qt::Vertical);
-
     /* test for the evolution dbus interface */
     // TODO find out why evolution does not expose dbus on a kde session here
 //    QDBusInterface evolutionDBusTest( "org.freedesktop.Notification",
@@ -160,7 +146,10 @@ void IncomingMsg::initLayout()
 //            mLayout->addItem(mEvolutionLayout);
 //        }
 //    }
+}
 
+void IncomingMsg::initKMailLayout()
+{
     /* test for the kmail dbus interface */
     if (mShowKMail) {
         QDBusInterface kmailDBusTest("org.kde.kmail", "/KMail", "org.freedesktop.DBus.Introspectable");
@@ -183,9 +172,10 @@ void IncomingMsg::initLayout()
                 mKMailIconLabel->setMinimumWidth(32);
                 mKMailIconLabel->setMinimumHeight(32);
                 KIconEffect effect;
-                mKMailIconLabel->nativeWidget()->setPixmap(
-                    effect.apply(icon.pixmap(32, 32), KIconEffect::ToGray, 1, QColor(), QColor(), true)
-                );
+                mKMailIconLabel->nativeWidget()->setPixmap(effect.apply(icon.pixmap(32, 32),
+                                                                        KIconEffect::ToGray, 1,
+                                                                        QColor(), QColor(), true)
+                                                           );
 
                 mKMailLayout->addItem(mKMailIconLabel);
                 mKMailLayout->addItem(mKMailLabel);
@@ -195,7 +185,10 @@ void IncomingMsg::initLayout()
             }
         }
     }
+}
 
+void IncomingMsg::initXChatLayout()
+{
     /* test for the xchat dbus interface */
     // do not really understand how this interface works
     // got this working code from http://arstechnica.com/reviews/hardware/tux-droid-review.ars/3
@@ -230,7 +223,10 @@ void IncomingMsg::initLayout()
             }
         }
     }
+}
 
+void IncomingMsg::initKopeteLayout()
+{
     /* test for the kopete dbus interface */
     if (mShowKopete) {
         QDBusInterface kopeteDBusTest("org.kde.kopete", "/kopete", "org.freedesktop.DBus.Introspectable");
@@ -265,7 +261,10 @@ void IncomingMsg::initLayout()
             }
         }
     }
+}
 
+void IncomingMsg::initPidginLayout()
+{
     /* test for the pidgin dbus interface */
     // FIXME introspect does not work here with qdbus trying sth. else
     if (mShowPidgin) {
@@ -302,13 +301,131 @@ void IncomingMsg::initLayout()
             }
         }
     }
+}
+
+void IncomingMsg::initQutIMLayout()
+{
+    if (mShowQutIM) {
+        QDBusInterface qutimDBusTest("org.qutim", "/ChatLayer", "org.qutim.ChatLayer");
+        QDBusReply<QList<QDBusObjectPath> > sessionsReply = qutimDBusTest.call("sessions");
+        if (!sessionsReply.isValid())
+            kDebug() << "qutIM DBus interface test error: " << sessionsReply.error();
+        else {
+            QList<QDBusObjectPath> sessions = sessionsReply.value();
+            for (int i = 0; i < sessions.size(); i++) {
+                QDBusInterface qutimSession("org.qutim", sessions.at(i).path(),
+                                            "org.freedesktop.DBus.Properties");
+                QDBusMessage msg = qutimSession.call("Get", "org.qutim.ChatSession", "unread");
+                slotNewQutIM(msg, sessions.at(i).path());
+            }
+            QDBusConnection mDBus = QDBusConnection::sessionBus();
+            if (!mDBus.connect("org.qutim", QString(),
+                               "org.qutim.ChatSession", "unreadChanged",
+                               this, SLOT(slotNewQutIM(QDBusMessage))))
+                kDebug() << "Could not connect to qutIM on DBus.";
+            else {
+                mQutIMLayout = new QGraphicsLinearLayout(Qt::Horizontal);
+                mQutIMLabel = new Plasma::Label(this);
+                mQutIMIconLabel = new Plasma::Label(this);
+                mQutIMIconLabel->setMinimumWidth(32);
+                mQutIMIconLabel->setMinimumHeight(32);
+
+                updateQutIMStatus(false);
+
+                mQutIMLayout->addItem(mQutIMIconLabel);
+                mQutIMLayout->addItem(mQutIMLabel);
+                mQutIMLayout->setAlignment(mQutIMLabel, Qt::AlignLeft);
+
+                mLayout->addItem(mQutIMLayout);
+            }
+        }
+    }
+}
+
+void IncomingMsg::updateQutIMStatus(bool saveIcon)
+{
+    if (!saveIcon) {
+        KIcon icon("qutim");
+        if (mQutIUnreadCount == 0) {
+            KIconEffect effect;
+            mQutIMIconLabel->nativeWidget()->setPixmap(
+                effect.apply(icon.pixmap(32, 32), KIconEffect::ToGray, 1, QColor(), QColor(), true)
+            );
+        } else {
+            mQutIMIconLabel->nativeWidget()->setPixmap(icon.pixmap(32, 32));
+        }
+    }
+    if (mQutIUnreadCount > 0) {
+        kDebug() << "You have " << mQutIUnreadCount << " new qutIM message(s).";
+        mQutIMLabel->setText(i18np("You have a new qutIM message.",
+                                   "You have %1 new qutIM messages.",
+                                   mQutIUnreadCount));
+    } else {
+        kDebug() << "No new qutIM messages.";
+        mQutIMLabel->setText(i18n("No new qutIM messages."));
+    }
+}
+
+void IncomingMsg::clearLayout()
+{
+    delete mKMailLayout;
+    mKMailLayout = NULL;
+    delete mKMailIconLabel;
+    mKMailIconLabel = NULL;
+    delete mKMailLabel;
+    mKMailLabel = NULL;
+
+    delete mXChatLayout;
+    mXChatLayout = NULL;
+    delete mXChatIconLabel;
+    mXChatIconLabel = NULL;
+    delete mXChatLabel;
+    mXChatLabel = NULL;
+
+    delete mKopeteLayout;
+    mKopeteLayout = NULL;
+    delete mKopeteIconLabel;
+    mKopeteIconLabel = NULL;
+    delete mKopeteLabel;
+    mKopeteLabel = NULL;
+
+    delete mPidginLayout;
+    mPidginLayout = NULL;
+    delete mPidginIconLabel;
+    mPidginIconLabel = NULL;
+    delete mPidginLabel;
+    mPidginLabel = NULL;
+    
+    delete mQutIMLayout;
+    mQutIMLayout = NULL;
+    delete mQutIMIconLabel;
+    mQutIMIconLabel = NULL;
+    delete mQutIMLabel;
+    mQutIMLabel = NULL;
+    mQutIMUnread.clear();
+    mQutIUnreadCount = 0;
+
+    delete mErrorLabel;
+    mErrorLabel = NULL;
+}
+
+void IncomingMsg::initLayout()
+{
+    mLayout = new QGraphicsLinearLayout(Qt::Vertical);
+
+    //initEvolutionLayout();
+    initKMailLayout();
+    initXChatLayout();
+    initKopeteLayout();
+    initPidginLayout();
+    initQutIMLayout();
 
     if (!mLayout->count()) {
-        Plasma::Label *errorLabel = new Plasma::Label();
-        errorLabel->setText(i18n("No running messaging apps found. Supported apps are %1, %2, %3, %4.",
-                                 QString("KMail"), QString("XChat"), QString("Kopete"),
-                                 QString("Pidgin")));
-        mLayout->addItem(errorLabel);
+        mErrorLabel = new Plasma::Label();
+        mErrorLabel->setText(i18n("No running messaging apps found. Supported apps are %1, %2, %3, %4, %5.",
+                                  QString("KMail"), QString("XChat"), QString("Kopete"),
+                                  QString("Pidgin"), QString("qutIM")));
+        mLayout->addItem(mErrorLabel);
     }
 
     setLayout(mLayout);
@@ -324,6 +441,7 @@ void IncomingMsg::createConfigurationInterface(KConfigDialog *dialog)
     ui.showXChat->setChecked(cg.readEntry("showXChat", true));
     ui.showKopete->setChecked(cg.readEntry("showKopete", true));
     ui.showPidgin->setChecked(cg.readEntry("showPidgin", true));
+    ui.showQutIM->setChecked(cg.readEntry("showQutIM", true));
 
     connect(dialog, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
     connect(dialog, SIGNAL(okClicked()), this, SLOT(configAccepted()));
@@ -338,29 +456,19 @@ void IncomingMsg::configAccepted()
     mShowXChat = ui.showXChat->isChecked();
     mShowKopete = ui.showKopete->isChecked();
     mShowPidgin = ui.showPidgin->isChecked();
+    mShowQutIM = ui.showQutIM->isChecked();
 
     KConfigGroup cg = config();
     cg.writeEntry("showKMail", ui.showKMail->isChecked());
     cg.writeEntry("showXChat", ui.showXChat->isChecked());
     cg.writeEntry("showKopete", ui.showKopete->isChecked());
     cg.writeEntry("showPidgin", ui.showPidgin->isChecked());
+    cg.writeEntry("showQutIM", ui.showQutIM->isChecked());
 
+    clearLayout();
     initLayout();
 }
 
-void IncomingMsg::constraintsEvent(Plasma::Constraints constraints)
-{
-    if (mLayout && (constraints & Plasma::SizeConstraint ||
-                    constraints & Plasma::FormFactorConstraint)) {
-        QRectF layoutRectF = mLayout->geometry();
-        QRectF appletRectF = geometry();
-        if (layoutRectF.height() > appletRectF.height()) {
-            appletRectF.setHeight(layoutRectF.height() + 10);
-            setGeometry(appletRectF);
-        }
-    }
-}
-
 void IncomingMsg::slotNewEvolutionMail()
 {
     KIcon icon("evolution");
@@ -382,6 +490,42 @@ void IncomingMsg::slotNewXChatIM()
     mXChatLabel->setText(i18n("You have new XChat messages."));
 }
 
+void IncomingMsg::slotNewQutIM(const QDBusMessage &msg, QString path)
+{
+    if (path.isNull())
+        path = msg.path();
+    bool hasUnread = mQutIUnreadCount > 0;
+    QVariant var = msg.arguments().value(0);
+    if (var.canConvert<QDBusVariant>())
+        var = var.value<QDBusVariant>().variant();
+    int currentCount = mQutIMUnread.value(path, 0);
+    int count = 0;
+    const QDBusArgument arg = qvariant_cast<QDBusArgument>(var);
+    // Parse aa{sv} value
+    arg.beginArray();
+    while (!arg.atEnd()) {
+        arg.beginMap();
+        while (!arg.atEnd()) {
+            QString key;
+            QVariant value;
+            arg.beginMapEntry();
+            arg >> key >> value;
+            arg.endMapEntry();
+        }
+        arg.endMap();
+        count++;
+    }
+    arg.endArray();
+    mQutIUnreadCount -= currentCount;
+    mQutIUnreadCount += count;
+    if (count > 0)
+        mQutIMUnread.insert(path, count);
+    else
+        mQutIMUnread.remove(path);
+    if (mQutIMLayout)
+        updateQutIMStatus(hasUnread == (mQutIUnreadCount > 0));
+}
+
 void IncomingMsg::slotNewKopeteIM(const QString& contactId)
 {
     QDBusInterface kopeteDBusTest("org.kde.kopete", "/Kopete", "org.kde.Kopete");
