@@ -22,18 +22,22 @@
 #include "Helpers.h"
 #include <QPainter>
 
+#include <Plasma/Theme>
 
 ArrowTopKey::ArrowTopKey(QPoint relativePosition, QSize relativeSize) :
-        FuncKey(relativePosition, relativeSize, Helpers::keysymToKeycode(XK_Up), QString()){
+        FuncKey(relativePosition, relativeSize, Helpers::keysymToKeycode(XK_Up), QString())
+{
 
 }
 
-void ArrowTopKey::paint(QPainter *painter){
+void ArrowTopKey::paint(QPainter *painter)
+{
     FuncKey::paint(painter);
 
     painter->save();
     setUpPainter(painter);
     painter->setRenderHint(QPainter::Antialiasing);
+    painter->setPen(Plasma::Theme::defaultTheme()->color(Plasma::Theme::ButtonTextColor));
 	painter->rotate(90);
 	paintArrow(painter);
     painter->restore();
