@@ -30,43 +30,42 @@ class Layout;
 
 class PanelIcon: public Plasma::PopupApplet {
     Q_OBJECT
+
 public:
-	PanelIcon(QObject *parent, const QVariantList &args);
-	virtual ~PanelIcon();
-	QGraphicsWidget *graphicsWidget();
-
-	/*
-	 * Adds the layout switching entry to the context menu
-	 */
-
-    //QList<QAction*> contextualActions();
+    PanelIcon(QObject *parent, const QVariantList &args);
+    virtual ~PanelIcon();
 
+    void configChanged();
+    void constraintsEvent(Plasma::Constraints constraints);
+    QGraphicsWidget *graphicsWidget();
 
 public Q_SLOTS:
     void configAccepted();
-    void configChanged(QString name);
     /**
       * Initialize the keyboard with the configured layout
       */
     void initKeyboard();
+    void layoutNameChanged(const QString &name);
+
 
 protected:
     void createConfigurationInterface(KConfigDialog *parent);
-	/*
-	 * Clears all pressed keys when keyboard is cloased
-	 */
-	virtual void popupEvent(bool show);
+
+    /*
+     * Clears all pressed keys when keyboard is cloased
+     */
+    virtual void popupEvent(bool show);
 
 private:
     void init();
-    void initKeyboard(QString layoutFile);
-    void saveLayout(QString path);
+    void initKeyboard(const QString &layoutFile);
+    void saveLayout(const QString &path);
 
 private:
     QString m_layout;
     QList<Layout*> m_layouts;
-	PlasmaboardWidget *m_plasmaboard;
-	const QVariantList args;
+    PlasmaboardWidget *m_plasmaboard;
+    const QVariantList args;
 
     Ui::config ui;
 
