@@ -36,20 +36,28 @@
 #include "SwitchKey.h"
 #include "TabKey.h"
 
+#include <QDBusConnection>
 #include <QFile>
-#include <QStyleOptionGraphicsItem>
 #include <QGraphicsSceneResizeEvent>
-#include <QTimer>
 #include <QPainter>
+#include <QSignalMapper>
+#include <QStyleOptionGraphicsItem>
+#include <QTimer>
+#include <plasma/containment.h>
+#include <plasma/corona.h>
 #include <plasma/theme.h>
 
 #include "Helpers.h"
 
+#define REPEAT_TIMER 1500
+#define STICKY_TIMER 500
+
 QChar Helpers::mapXtoUTF8[0xffff+1];
 int Helpers::keysymsPerKeycode;
 
-PlasmaboardWidget::PlasmaboardWidget(QGraphicsWidget *parent)
-    : Plasma::Applet(parent)
+PlasmaboardWidget::PlasmaboardWidget(Plasma::PopupApplet *parent)
+    : QGraphicsWidget(parent),
+      m_applet(parent)
 {
     setPreferredSize(500, 200);
     setMinimumSize(200,100);
@@ -57,7 +65,6 @@ PlasmaboardWidget::PlasmaboardWidget(QGraphicsWidget *parent)
     setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint | Qt::X11BypassWindowManagerHint);
     setFocusPolicy(Qt::NoFocus);
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-    setBackgroundHints(Plasma::Applet::TranslucentBackground);
 
     Helpers::buildUp();
     m_isLevel2 = false;
@@ -77,15 +84,20 @@ PlasmaboardWidget::PlasmaboardWidget(QGraphicsWidget *parent)
     m_activeFrame->setImagePath("widgets/button");
     m_activeFrame->setElementPrefix("pressed");
 
-    m_engine = dataEngine("keystate");
+    m_engine = m_applet->dataEngine("keystate");
     if(m_engine){
-        m_engine -> connectAllSources(this);
+        m_engine->connectAllSources(this);
     }
 
+    m_signalMapper = new QSignalMapper(this);
+    connect(m_signalMapper, SIGNAL(mapped(int)), this, SLOT(stickyKey_Mapper(int)));
+
     m_repeatTimer = new QTimer(this);
     connect(m_repeatTimer, SIGNAL(timeout()), this, SLOT(repeatKeys()));
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeChanged()));
-
+    
+    QDBusConnection dbus = QDBusConnection::sessionBus();
+    dbus.connect("org.kde.keyboard", "/Layouts", "org.kde.KeyboardLayouts", "currentLayoutChanged", this, SLOT(relabelKeys()));
 }
 
 
@@ -122,50 +134,14 @@ void PlasmaboardWidget::clearTooltip()
     m_tooltip->hide();
 }
 
-FuncKey* PlasmaboardWidget::createFunctionKey(QPoint &point, QSize &size, QString action)
+FuncKey* PlasmaboardWidget::createFunctionKey(const QPoint &point, const QSize &size, const QString &action)
 {
-
-    if(action == "ALT"){
-        StickyKey *k = new StickyKey(point, size, Helpers::keysymToKeycode(XK_Alt_L), QString(i18n("Alt")));
-        m_altKeys << k;
-        return k;
-    }
-    else if(action == "ALTGR"){
-        StickyKey *k = new StickyKey(point, size, Helpers::keysymToKeycode(XK_ISO_Level3_Shift), QString( i18nc("The Alt Gr key on a keyboard", "Alt Gr")));
-        m_altgrKeys << k;
-        return k;
-    }
-    else if(action == "BACKSPACE")
+    if(action == "BACKSPACE")
         return new BackspaceKey(point, size);
-    else if(action == "CAPSLOCK"){
-        CapsKey *k = new CapsKey(point, size);
-        m_capsKeys << k;
-        return k;
-    }
-    else if(action == "CONTROLLEFT"){
-        StickyKey *k = new StickyKey(point, size, Helpers::keysymToKeycode(XK_Control_L), QString(i18nc("The Ctrl key on a keyboard", "Ctrl")));
-        m_ctlKeys << k;
-        return k;
-    }
-    else if(action == "CONTROLRIGHT"){
-        StickyKey *k = new StickyKey(point, size, Helpers::keysymToKeycode(XK_Control_R), QString(i18nc("The Ctrl key on a keyboard", "Ctrl")));
-        m_ctlKeys << k;
-        return k;
-    }
     else if(action == "ENTER")
         return new EnterKey(point, size);
-    else if(action == "SHIFT") {
-        ShiftKey *k = new ShiftKey(point, size);
-        m_shiftKeys << k;
-        return k;
-    }
     else if(action == "SPACE")
-        return new FuncKey(point, size, Helpers::keysymToKeycode(XK_space), QString());
-    else if(action == "SUPERLEFT"){
-        StickyKey *k = new StickyKey(point, size, Helpers::keysymToKeycode(XK_Super_L), QString( i18nc("The super (windows) key on a keyboard", "Super")));
-        m_superKeys << k;
-        return k;
-    }
+        return new FuncKey(point, size, Helpers::keysymToKeycode(XK_space), QString());    
     else if(action == "TAB")
         return new TabKey(point, size);
     else if(action == "ESCAPE")
@@ -251,7 +227,48 @@ FuncKey* PlasmaboardWidget::createFunctionKey(QPoint &point, QSize &size, QStrin
     else if(action == "KEYPAD9")
         return new FuncKey(point, size, Helpers::keysymToKeycode(XK_KP_9), QString(i18nc("9 key on the keypad", "9")));
     else if(action == "KEYPAD0")
-        return new FuncKey(point, size, Helpers::keysymToKeycode(XK_KP_0), QString(i18nc("0 key on the keypad", "0")));
+        return new FuncKey(point, size, Helpers::keysymToKeycode(XK_KP_0), QString(i18nc("0 key on the keypad", "0")));    
+    else
+        return new FuncKey(point, size, Helpers::keysymToKeycode(XK_space), QString("Unkown"));
+}
+
+FuncKey* PlasmaboardWidget::createStickyKey(const QPoint &point, const QSize &size, const QString &action)
+{
+    if(action == "ALT"){
+        StickyKey *k = new StickyKey(point, size, Helpers::keysymToKeycode(XK_Alt_L), QString(i18n("Alt")));
+        m_altKeys << k;
+        return k;
+    }
+    else if(action == "ALTGR"){
+        StickyKey *k = new StickyKey(point, size, Helpers::keysymToKeycode(XK_ISO_Level3_Shift), QString( i18nc("The Alt Gr key on a keyboard", "Alt Gr")));
+        m_altgrKeys << k;
+        return k;
+    }
+    else if(action == "CAPSLOCK"){
+        CapsKey *k = new CapsKey(point, size);
+        m_capsKeys << k;
+        return k;
+    }
+    else if(action == "CONTROLLEFT"){
+        StickyKey *k = new StickyKey(point, size, Helpers::keysymToKeycode(XK_Control_L), QString(i18nc("The Ctrl key on a keyboard", "Ctrl")));
+        m_ctlKeys << k;
+        return k;
+    }
+    else if(action == "CONTROLRIGHT"){
+        StickyKey *k = new StickyKey(point, size, Helpers::keysymToKeycode(XK_Control_R), QString(i18nc("The Ctrl key on a keyboard", "Ctrl")));
+        m_ctlKeys << k;
+        return k;
+    }
+    else if(action == "SHIFT") {
+        ShiftKey *k = new ShiftKey(point, size);
+        m_shiftKeys << k;
+        return k;
+    }
+    else if(action == "SUPERLEFT"){
+        StickyKey *k = new StickyKey(point, size, Helpers::keysymToKeycode(XK_Super_L), QString( i18nc("The super (windows) key on a keyboard", "Super")));
+        m_superKeys << k;
+        return k;
+    }
     else if(action == "SWITCH"){
         SwitchKey* key = new SwitchKey(point, size, Helpers::keysymToKeycode(XK_VoidSymbol), this);
         m_switchKeys << key;
@@ -313,11 +330,11 @@ void PlasmaboardWidget::dataUpdated(const QString &sourceName, const Plasma::Dat
 
 void PlasmaboardWidget::deleteKeys()
 {
-    qDeleteAll(m_funcKeys);
-    m_funcKeys.clear();
+    qDeleteAll(m_keys);
 
-    qDeleteAll(m_alphaKeys);
     m_alphaKeys.clear();
+    m_funcKeys.clear();
+    m_specialKeys.clear();
 
     m_keys.clear();
     m_altKeys.clear();
@@ -431,7 +448,10 @@ void PlasmaboardWidget::initKeyboard(const QString &file)
                     }
                 }
                 else if(m_xmlReader.name() == "fkey"){
-                    m_funcKeys << createFunctionKey(currentPoint, currentSize, m_xmlReader.attributes().value("action").toString());
+                    m_specialKeys << createFunctionKey(currentPoint, currentSize, m_xmlReader.attributes().value("action").toString());
+                }
+                else if(m_xmlReader.name() == "skey"){
+                    m_funcKeys << createStickyKey(currentPoint, currentSize, m_xmlReader.attributes().value("action").toString());
                 }
 
                 m_xmlReader.skipCurrentElement();
@@ -450,6 +470,9 @@ void PlasmaboardWidget::initKeyboard(const QString &file)
     Q_FOREACH(BoardKey* key, m_funcKeys){
         m_keys << key;
     }
+    Q_FOREACH(BoardKey* key, m_specialKeys){
+        m_keys << key;
+    }
 
     delete fileP;
 }
@@ -490,7 +513,7 @@ void PlasmaboardWidget::mouseMoveEvent ( QGraphicsSceneMouseEvent * event )
         }
     }
 
-    Plasma::Applet::mouseMoveEvent(event);
+    QGraphicsWidget::mouseMoveEvent(event);
 }
 
 void PlasmaboardWidget::mousePressEvent ( QGraphicsSceneMouseEvent * event )
@@ -502,7 +525,7 @@ void PlasmaboardWidget::mousePressEvent ( QGraphicsSceneMouseEvent * event )
             return;
         }
     }
-    Plasma::Applet::mousePressEvent(event);
+    QGraphicsWidget::mousePressEvent(event);
 }
 
 void PlasmaboardWidget::mouseReleaseEvent ( QGraphicsSceneMouseEvent * event )
@@ -514,21 +537,15 @@ void PlasmaboardWidget::mouseReleaseEvent ( QGraphicsSceneMouseEvent * event )
             return;
         }
     }
-    Plasma::Applet::mouseReleaseEvent(event);
+    QGraphicsWidget::mouseReleaseEvent(event);
 }
 
-/*bool PlasmaboardWidget::event ( QEvent * event )
-{
-    qDebug() << event->type();
-    return QGraphicsWidget::event(event);
-}*/
-
 void PlasmaboardWidget::paint(QPainter *p,
                               const QStyleOptionGraphicsItem *option,
                               QWidget* widget)
 {
     Q_UNUSED(widget);
-    qDebug() << "Painting: " << option->exposedRect;
+    //qDebug() << "Painting: " << option->exposedRect;
     //Plasma::Containment::paint(p, option, widget);
 
     p->setBrush(Plasma::Theme::defaultTheme()->color(Plasma::Theme::ButtonTextColor));
@@ -548,7 +565,7 @@ void PlasmaboardWidget::press(BoardKey *key)
     m_pressedList << key;    
     update(key->rect());
     setTooltip(key);
-    m_repeatTimer->start(1500);
+    m_repeatTimer->start(REPEAT_TIMER);
 }
 
 void PlasmaboardWidget::press(FuncKey *key)
@@ -567,7 +584,6 @@ void PlasmaboardWidget::refreshKeys()
         key->updateDimensions(factor_x, factor_y);
         key->setPixmap(getFrame(key->size()));
     }
-
 }
 
 void PlasmaboardWidget::relabelKeys()
@@ -580,17 +596,21 @@ void PlasmaboardWidget::relabelKeys()
 
 void PlasmaboardWidget::release(BoardKey *key)
 {    
-    key->released();
-    key->setPixmap(getFrame(key->size()));
-    m_pressedList.removeAll(key);
-    update(key->rect());
-    clearTooltip();
-
-    if(m_alphaKeys.contains((AlphaNumKey*) key)){
+    key->released(); // trigger X-unpress event done by key
+    m_pressedList.removeAll(key);    
+    clearTooltip(); // remove displayed tooltip
+    if(m_alphaKeys.contains((AlphaNumKey*) key) || m_specialKeys.contains((FuncKey*) key)){
         reset();
     }
-
     m_repeatTimer->stop();
+
+    int id = qrand();
+    m_stickyKeys[id] = key;
+
+    QTimer* timer = new QTimer(this);    
+    connect(timer, SIGNAL(timeout()), m_signalMapper, SLOT(map()));
+    m_signalMapper->setMapping(timer, id);
+    timer->start(STICKY_TIMER);
 }
 
 void PlasmaboardWidget::repeatKeys()
@@ -644,13 +664,38 @@ void PlasmaboardWidget::setTooltip(BoardKey* key)
     QString label = key->label();
     if(label.size() > 0) {
         m_tooltip -> setText( key->label() );
-        m_tooltip -> move( popupPosition( key->size()*2 ) + key->position() - QPoint(key->size().width()/2, 0) );
         m_tooltip -> resize( key->size()*2 );
+
+        Plasma::Containment *c = m_applet->containment();
+        if(c){
+            Plasma::Corona *corona = c->corona();
+            if(corona){
+		if(m_applet->location() == Plasma::TopEdge) {
+		  m_tooltip -> move( corona->popupPosition(this, key->size()*2, Qt::AlignLeft) + QPoint(key->position().x() - key->size().width()/2, 0) - QPoint(0, size().height() - key->position().y() - key->size().height()) );
+		}
+		else {
+		  m_tooltip -> move( corona->popupPosition(this, key->size()*2, Qt::AlignLeft) + key->position() - QPoint(key->size().width()/2, 0) );
+		}
+            }
+        }
+
         m_tooltip -> show();
+	m_tooltip -> raise();
     }
 }
 
-void PlasmaboardWidget::switchAlternative(bool alt){
+void PlasmaboardWidget::stickyKey_Mapper(int id)
+{
+    BoardKey* key = m_stickyKeys[id];
+    key->setPixmap(getFrame(key->size()));
+    update(key->rect());
+
+    delete (m_signalMapper->mapping(id)); // delete the QTimer
+    m_stickyKeys.remove(id);
+}
+
+void PlasmaboardWidget::switchAlternative(bool alt)
+{
     Q_FOREACH(DualKey* key, m_dualKeys){
         key->setAlternative(alt);
     }
