@@ -23,22 +23,35 @@
 #include <QGraphicsLinearLayout>
 #include <QGraphicsProxyWidget>
 #include <QLabel>
+#include <QRegExp>
 #include <KComboBox>
 #include <KLineEdit>
-#include <Plasma/ComboBox>
 #include <Plasma/LineEdit>
 #include <Plasma/Label>
 #include <Plasma/Frame>
-#include "conversion/converter.h"
-#include "conversion/unitcategory.h"
+#include <KUnitConversion/UnitCategory>
 
-Q_DECLARE_METATYPE(const Conversion::Unit*)
-Q_DECLARE_METATYPE(Conversion::UnitCategory*)
+using namespace KUnitConversion;
+
+ComboBox::ComboBox(QGraphicsWidget* parent)
+    : Plasma::ComboBox(parent)
+{
+}
+
+void ComboBox::mousePressEvent(QGraphicsSceneMouseEvent* event)
+{
+    emit mousePressed();
+    Plasma::ComboBox::mousePressEvent(event);
+}
+
+Q_DECLARE_METATYPE(UnitPtr)
+Q_DECLARE_METATYPE(UnitCategory*)
 
 UnitConverter::UnitConverter(QObject *parent, const QVariantList &args)
 : Plasma::PopupApplet(parent, args)
-, m_widget(0)
+, m_widget(0), m_bCalculateReverse(false)
 {
+    KGlobal::locale()->insertCatalog("libconversion");
     setAspectRatioMode(Plasma::IgnoreAspectRatio);
     setPopupIcon("accessories-calculator");
     resize(400, 300);
@@ -60,22 +73,26 @@ void UnitConverter::init()
 void UnitConverter::sltCategoryChanged(int index)
 {
     Q_UNUSED(index);
-    Conversion::UnitCategory* category =
-            m_pCmbCategory->nativeWidget()->itemData(index).value<Conversion::UnitCategory*>();
-    QList<Conversion::Unit*> units = category->units();
-    Conversion::Unit* defaultUnit = category->defaultUnit();
+    UnitCategory* category =
+            m_pCmbCategory->nativeWidget()->itemData(index).value<UnitCategory*>();
+    QList<UnitPtr> units = category->units();
+    UnitPtr defaultUnit = category->defaultUnit();
     m_pCmbUnit1->clear();
     m_pCmbUnit2->clear();
     int i = 0;
-    foreach (const Conversion::Unit* unit, units) {
-        m_pCmbUnit1->nativeWidget()->addItem(unit->singular(), QVariant::fromValue(unit));
-        m_pCmbUnit2->nativeWidget()->addItem(unit->singular(), QVariant::fromValue(unit));
+    foreach (const UnitPtr& unit, units) {
+        m_pCmbUnit1->nativeWidget()->addItem(QString("%1 (%2)")
+                .arg(unit->description()).arg(unit->symbol()), QVariant::fromValue(unit));
+        m_pCmbUnit2->nativeWidget()->addItem(QString("%1 (%2)")
+                .arg(unit->description()).arg(unit->symbol()), QVariant::fromValue(unit));
         if (unit == defaultUnit) {
             m_pCmbUnit1->nativeWidget()->setCurrentIndex(i);
             m_pCmbUnit2->nativeWidget()->setCurrentIndex(i);
         }
         ++i;
     }
+    m_pCmbUnit1->nativeWidget()->model()->sort(0);
+    m_pCmbUnit2->nativeWidget()->model()->sort(0);
     if (!category->description().isEmpty()) {
         m_pInfo->setText(QString("<a href=\"%2\">%1</a>")
                 .arg(category->description()).arg(category->url().prettyUrl()));
@@ -88,26 +105,67 @@ void UnitConverter::sltCategoryChanged(int index)
 void UnitConverter::sltUnitChanged(int index)
 {
     Q_UNUSED(index);
-    calculate();
+    if ( m_bCalculateReverse ) {
+        calculateReverse();
+    }
+    else {
+        calculate();
+    }
 }
 
 void UnitConverter::sltValueChanged(const QString &sNewValue)
 {
     Q_UNUSED(sNewValue);
+    m_bCalculateReverse = false; // store calculation direction
     calculate();
 }
 
+void UnitConverter::sltValueChangedReverse(const QString &sNewValue)
+{
+    Q_UNUSED(sNewValue);
+    m_bCalculateReverse = true; // store calculation direction
+    calculateReverse();
+}
+
+/// Calculates from left to right
 void UnitConverter::calculate()
 {
-    const Conversion::Unit* in = m_pCmbUnit1->nativeWidget()->itemData(
-            m_pCmbUnit1->nativeWidget()->currentIndex()).value<const Conversion::Unit*>();
-    const Conversion::Unit* out = m_pCmbUnit2->nativeWidget()->itemData(
-            m_pCmbUnit2->nativeWidget()->currentIndex()).value<const Conversion::Unit*>();
-    if (in && out) {
-        Conversion::Value dblValueIn(m_pTxtValue1->text().toDouble(), in);
-        Conversion::Value dblValueOut = in->category()->convert(dblValueIn, out->symbol());
+    UnitPtr in = m_pCmbUnit1->nativeWidget()->itemData(
+            m_pCmbUnit1->nativeWidget()->currentIndex()).value<UnitPtr>();
+    UnitPtr out = m_pCmbUnit2->nativeWidget()->itemData(
+            m_pCmbUnit2->nativeWidget()->currentIndex()).value<UnitPtr>();
+    if (!in.isNull() && !out.isNull()) {
+        Value dblValueIn(m_pTxtValue1->text().toDouble(), in);
+        Value dblValueOut = dblValueIn.convertTo(out->id());
+        QRegExp decimalCheck("^\\d+\\.0$");
+        QRegExp onlyDecimal("^\\d+$");
+        if(decimalCheck.exactMatch(m_pTxtValue1->text()) && onlyDecimal.exactMatch(QString::number(dblValueOut.number()))) {
+           QString addZero = QString::number(dblValueOut.number()) + ".0";
+           m_pTxtValue2->setText(addZero);
+        } else {
+           m_pTxtValue2->setText(QString::number(dblValueOut.number(), 'g', 15));
+        }
+    }
+}
 
-        m_pTxtValue2->setText(QString::number(dblValueOut.number()));
+/// Calculates from right to left
+void UnitConverter::calculateReverse()
+{
+    UnitPtr in = m_pCmbUnit2->nativeWidget()->itemData(
+            m_pCmbUnit2->nativeWidget()->currentIndex()).value<UnitPtr>();
+    UnitPtr out = m_pCmbUnit1->nativeWidget()->itemData(
+            m_pCmbUnit1->nativeWidget()->currentIndex()).value<UnitPtr>();
+    if (!in.isNull() && !out.isNull()) {
+        Value dblValueIn(m_pTxtValue2->text().toDouble(), in);
+        Value dblValueOut = dblValueIn.convertTo(out->id());
+        QRegExp decimalCheck("^\\d+\\.0$");
+        QRegExp onlyDecimal("^\\d+$");
+        if(decimalCheck.exactMatch(m_pTxtValue2->text()) && onlyDecimal.exactMatch(QString::number(dblValueOut.number()))) {
+           QString addZero = QString::number(dblValueOut.number()) + ".0";
+           m_pTxtValue1->setText(addZero);
+        } else {
+           m_pTxtValue1->setText(QString::number(dblValueOut.number(), 'g', 15));
+        }
     }
 }
 
@@ -122,11 +180,14 @@ QGraphicsWidget *UnitConverter::graphicsWidget()
         Plasma::Label *pLabel = new Plasma::Label(this);
         pLabel->nativeWidget()->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
         pLabel->setText(i18n("Convert:"));
-        m_pCmbCategory = new Plasma::ComboBox(this);
+        m_pCmbCategory = new ComboBox(this);
+        connect(m_pCmbCategory, SIGNAL(mousePressed()), this, SLOT(raise()));
         m_pCmbCategory->setZValue(2);
 
-        m_pCmbUnit1 = new Plasma::ComboBox(this);
-        m_pCmbUnit2 = new Plasma::ComboBox(this);
+        m_pCmbUnit1 = new ComboBox(this);
+        m_pCmbUnit2 = new ComboBox(this);
+        connect(m_pCmbUnit1, SIGNAL(mousePressed()), this, SLOT(raise()));
+        connect(m_pCmbUnit2, SIGNAL(mousePressed()), this, SLOT(raise()));
         m_pCmbUnit1->setZValue(1);
         m_pCmbUnit2->setZValue(1);
         m_pTxtValue1 = new Plasma::LineEdit(this);
@@ -146,37 +207,45 @@ QGraphicsWidget *UnitConverter::graphicsWidget()
         pGridLayout->addItem(m_pInfo, 4, 0, 1, 2);
         pGridLayout->setRowStretchFactor(5, 1);
 
-        QList<Conversion::UnitCategory*> categories = Conversion::Converter::self()->categories();
-        foreach (Conversion::UnitCategory* category, categories) {
+        foreach (UnitCategory* category, m_converter.categories()) {
             m_pCmbCategory->nativeWidget()->addItem(category->name(), QVariant::fromValue(category));
         }
-        // Load previous values
-        KConfigGroup cg = config();
-        int category = cg.readEntry("category", 0);
-        m_pCmbCategory->nativeWidget()->setCurrentIndex(category);
-        sltCategoryChanged(category);
-        int unit1 = cg.readEntry("unit1", -1);
-        if (unit1 >= 0) {
-            m_pCmbUnit1->nativeWidget()->setCurrentIndex(unit1);
-        }
-        int unit2 = cg.readEntry("unit2", -1);
-        if (unit2 >= 0) {
-            m_pCmbUnit2->nativeWidget()->setCurrentIndex(unit2);
-        }
-        m_pTxtValue1->setText(cg.readEntry("value", "1.0"));
-        calculate();
+        m_pCmbCategory->nativeWidget()->model()->sort(0);
 
-        connect(m_pTxtValue1->nativeWidget(), SIGNAL(textChanged(const QString&)),
+        connect(m_pTxtValue1->nativeWidget(), SIGNAL(textEdited(const QString&)),
                 this, SLOT(sltValueChanged(const QString&)));
+        connect(m_pTxtValue2->nativeWidget(), SIGNAL(textEdited(const QString&)),
+                this, SLOT(sltValueChangedReverse(const QString&)));
         connect(m_pCmbCategory->nativeWidget(), SIGNAL(currentIndexChanged(int)),
                 this, SLOT(sltCategoryChanged(int)));
         connect(m_pCmbUnit1->nativeWidget(), SIGNAL(currentIndexChanged(int)),
                 this, SLOT(sltUnitChanged(int)));
         connect(m_pCmbUnit2->nativeWidget(), SIGNAL(currentIndexChanged(int)),
                 this, SLOT(sltUnitChanged(int)));
+
+        configChanged();
     }
     return m_widget;
 }
 
+void UnitConverter::configChanged()
+{
+    // Load previous values
+    KConfigGroup cg = config();
+    int category = cg.readEntry("category", 0);
+    m_pCmbCategory->nativeWidget()->setCurrentIndex(category);
+    sltCategoryChanged(category);
+    int unit1 = cg.readEntry("unit1", -1);
+    if (unit1 >= 0) {
+        m_pCmbUnit1->nativeWidget()->setCurrentIndex(unit1);
+    }
+    int unit2 = cg.readEntry("unit2", -1);
+    if (unit2 >= 0) {
+        m_pCmbUnit2->nativeWidget()->setCurrentIndex(unit2);
+    }
+    m_pTxtValue1->setText(cg.readEntry("value", "1"));
+    calculate();
+}
+
 #include "unitconverter.moc"
 
