@@ -60,7 +60,7 @@ void BackgroundListModel::reload()
 
 void BackgroundListModel::reload(const QStringList &selected)
 {
-    const QStringList dirs = KGlobal::dirs()->findDirs("wallpaper", "");
+    const QStringList dirs = KGlobal::dirs()->findDirs("wallpaper", QLatin1String( "" ));
     QList<Plasma::Package *> tmp;
 
     if (!m_packages.isEmpty()) {
@@ -121,7 +121,7 @@ QModelIndex BackgroundListModel::indexOf(const QString &path) const
     for (int i = 0; i < m_packages.size(); i++) {
         // packages will end with a '/', but the path passed in may not
         QString package = m_packages[i]->path();
-        if (package.at(package.length() - 1) == '/') {
+        if (package.at(package.length() - 1) == QLatin1Char( '/' )) {
             package.truncate(package.length() - 1);
         }
 
@@ -159,8 +159,8 @@ QSize BackgroundListModel::bestSize(Plasma::Package *package) const
     }
 
     KFileMetaInfo info(image, QString(), KFileMetaInfo::TechnicalInfo);
-    QSize size(info.item("http://freedesktop.org/standards/xesam/1.0/core#width").value().toInt(),
-               info.item("http://freedesktop.org/standards/xesam/1.0/core#height").value().toInt());
+    QSize size(info.item(QLatin1String( "http://freedesktop.org/standards/xesam/1.0/core#width" )).value().toInt(),
+               info.item(QLatin1String( "http://freedesktop.org/standards/xesam/1.0/core#height" )).value().toInt());
 
     //backup solution if strigi does not work
     if (size.width() == 0 || size.height() == 0) {
@@ -233,7 +233,7 @@ QVariant BackgroundListModel::data(const QModelIndex &index, int role) const
         QSize size = bestSize(b);
 
         if (size.isValid()) {
-            return QString("%1x%2").arg(size.width()).arg(size.height());
+            return QString(QLatin1String( "%1x%2" )).arg(size.width()).arg(size.height());
         }
 
         return QString();
@@ -308,7 +308,7 @@ void BackgroundFinder::start()
     progress->progressBar()->setRange(0, 0);
 
     QSet<QString> suffixes;
-    suffixes << "png" << "jpeg" << "jpg" << "svg" << "svgz";
+    suffixes << QLatin1String( "png" ) << QLatin1String( "jpeg" ) << QLatin1String( "jpg" ) << QLatin1String( "svg" ) << QLatin1String( "svgz" );
 
     QDir dir;
     dir.setFilter(QDir::AllDirs | QDir::Files | QDir::Hidden | QDir::Readable);
@@ -325,15 +325,15 @@ void BackgroundFinder::start()
             if (wp.isDir()) {
                 //kDebug() << "directory" << wp.fileName() << validPackages.contains(wp.fileName());
                 QString name = wp.fileName();
-                if (name == "." || name == "..") {
+                if (name == QLatin1String( "." ) || name == QLatin1String( ".." )) {
                     // do nothing
-                } else if(QFile::exists(wp.filePath() + "/metadata.desktop")) {
+                } else if(QFile::exists(wp.filePath() + QLatin1String( "/metadata.desktop" ))) {
                     Plasma::PackageStructure::Ptr structure = Plasma::Wallpaper::packageStructure(m_structureParent);
                     Plasma::Package pkg(wp.filePath(), structure);
 
                     if (pkg.isValid() && (!m_container || !m_container->contains(pkg.path()))) {
                         if (setLabel) {
-                            progress->setLabelText(i18n("Finding images for the wallpaper slideshow.") + "\n\n" +
+                            progress->setLabelText(i18n("Finding images for the wallpaper slideshow.") + QLatin1String( "\n\n" ) +
                                                    i18n("Adding wallpaper package in %1", name));
                         }
 
@@ -349,7 +349,7 @@ void BackgroundFinder::start()
             } else if (suffixes.contains(wp.suffix().toLower()) && (!m_container || !m_container->contains(wp.filePath()))) {
                 //kDebug() << "adding" << wp.filePath() << setLabel;
                 if (setLabel) {
-                    progress->setLabelText(i18n("Finding images for the wallpaper slideshow.") + "\n\n" +
+                    progress->setLabelText(i18n("Finding images for the wallpaper slideshow.") + QLatin1String( "\n\n" ) +
                                            i18n("Adding image %1", wp.filePath()));
                     setLabel = false;
                 }
